/* Vconvert - convert one type of velocity function to another

  FROM HISTORY:
  	n1= d1= o1= 	fast axis (time or depth) 
  	(n2=) 		intermediate axis
  	(n3=) 		slow axis
  FROM PAR:
	n1out=		Output fast axis; only can change if outpar != inpar.
	d1out=	 	Ignored if outpar = inpar.
	o1out=	
 	(label1=)
	oversamp=1	oversampling rate (default is no oversampling)
	lsinc=12	oversampling interpolator length
			oversampling used with inpar != outpar only.
	alpha=0.01	fraction of peak value at ends of gaussian taper
			on sinc interpolator for oversampling
  FROM HISTORY OR PAR:
	invelopt=	=1, velocity
	outvelopt=	=2, slowness
                        =3, slowness squared
                        =4, velocity squared

	intype=		=1, interval
	outtype=	=2, rms 

	inpar=		=1, function of time
	outpar=		=2, function of depth

  NOTE: Vrms(z) [type=2 and par=2] is not understood by this routine.

  NOTE: oversampling may improve v(z)<=>v(t) conversion if velocity
        field contains high wavenumber components.  It can introduce
        spurious high wavenumber jitter if the velocity field is smooth.
        Decreasing alpha may help.  Use with caution!
#
written October 1986 by Paul Fowler 
October, 1986 added type and par options -Paul Fowler
2/88 changed to newer oversampling subroutines -Paul Fowler
2/88 added velocity squared sampling (velopt=4) -Paul Fowler
2/88 banned Vrms(z) and rewrote vconv.f logic -Paul Fowler
2/88 changed oversampling default to =1 (no oversampling) -P Fowler
2/88 added user specifiable alpha parameter -P Fowler
*/
/* Keywords : velocity-conversion velocity slowness interval-velocity
 * Keywords : rms-velocity slowness-squared velocity-squared
 * Keywords : velocity-analysis
 */
#include <sep.main>
/*
 * sets up infd, outfd, instream, outstream, doc (SOURCE), pi, and 
 * #include <stdio.h>
 * #include <math.h>
 * #include <seplib.h>
 */
MAIN()
{
   int lsinc;		/* length of sinc interpolator			*/
   int n1in;		/* number of fast axis points on input		*/
   int n1out;		/* number of fast axis points on output		*/
   int n1pad;		/* number of fast axis points after oversampling */
   int n2;		/* number of intermediate axis points 		*/
   int n3;		/* number of slow axis points	 		*/
   int ovrsmp;		/* oversampling factor				*/
   int points;		/* ovrsmp-1					*/
   int vopt1;		/* input velocity sampling option		*/
   int vopt2;		/* output velocity sampling option		*/
   int vpar1;		/* input velocity parameter (time or depth)	*/
   int vpar2;		/* output velocity parameter (time or depth)	*/
   int vtype1;		/* input velocity type (interval or rms)	*/
   int vtype2;		/* output velocity type (interval or rms)	*/

   float alpha;		/* fraction of peak value at interpolator taper ends */
   float d1in;		/* input fast axis sampling interval		*/
   float d1out;		/* output fast axis sampling interval		*/
   float o1in;		/* input fast axis origin			*/
   float o1out;		/* output fast axis origin			*/
   float *data1;	/* input data array				*/
   float *data2;	/* output data array				*/
   float *osmpwt;	/* oversampling interpolator coefficients	*/
   float *work1;	/* work vector					*/
   float *work2;	/* work vector					*/
   float *work3;	/* work vector					*/
   float *zin;		/* converted values of input parameter		*/

   /* Input parameters from data history header and command line */

   getparameters( &n1in,   &n1out,  &n2,     &n3,     &d1in,
                  &d1out,  &o1in,   &o1out,  &vopt1,  &vopt2,
                  &vpar1,  &vpar2,  &vtype1, &vtype2, &lsinc,
                  &points, &n1pad,  &ovrsmp, &alpha             );

   /* Close data header file */

   hclose();

   /* Allocate space for work array */

   data1 = (float *) alloc( n1in * n2 * sizeof(float) );
   data2 = (float *) alloc( n1out * n2 * sizeof(float) );
   work1 = (float *) alloc( n1in * sizeof(float) );
   work2 = (float *) alloc( n1pad * sizeof(float) );
   work3 = (float *) alloc( n1out * sizeof(float) );
   zin = (float *) alloc( n1pad * sizeof(float) );
   osmpwt = (float *) alloc( lsinc * ovrsmp * sizeof(float) );

   /* perform velocity conversion */

   vconv_( &infd,   &outfd,  data1,   data2,  &n1in,
           &n1out,  &n2,     &n3,     &d1in,  &d1out,
           &o1in,   &o1out,  &vopt1,  &vopt2, &vpar1,
           &vpar2,  &vtype1, &vtype2, &lsinc, &points,
           zin,     osmpwt,  work1,   work2,  work3,
           &n1pad,  &ovrsmp, &alpha                       );

}

/*
 *  getparameters: gets values for all parameters from data processing 
 *  history file or from operating system command line for program
 *  Vconvert
 *  Note that all arguments are passed as pointers.
 *
 */

getparameters( pn1in,   pn1out,  pn2,     pn3,     pd1in,
               pd1out,  po1in,   po1out,  pvopt1,  pvopt2,
               pvpar1,  pvpar2,  pvtype1, pvtype2, plsinc,
               ppoints, pn1pad,  povrsmp, palpha             )

/* Arguments set on input: none
 * 
 * Arguments changed on output: all
 */

int *plsinc;	/* length of interpolator			*/
int *pn1in;	/* number of fast axis points on input		*/
int *pn1out;	/* number of fast axis points on output		*/
int *pn1pad;	/* number of fast axis points after oversampling */
int *pn2;	/* number of intermediate axis points 		*/
int *pn3;	/* number of slow axis points	 		*/
int *povrsmp;	/* oversampling factor				*/
int *ppoints;	/* ovrsmp-1					*/
int *pvopt1;	/* input velocity sampling option		*/
int *pvopt2;	/* output velocity sampling option		*/
int *pvpar1;	/* input velocity parameter (time or depth)	*/
int *pvpar2;	/* output velocity parameter (time or depth)	*/
int *pvtype1;	/* input velocity type (interval or rms)	*/
int *pvtype2;	/* output velocity type (interval or rms)	*/

float *palpha;	/* fractio of peak value at ends of interpolator taper */
float *pd1in;	/* input fast axis sampling interval		*/
float *pd1out;	/* output fast axis sampling interval		*/
float *po1in;	/* input fast axis origin			*/
float *po1out;	/* output fast axis origin			*/

{

   char verbose[20];	/* verbose option 				*/
   int oflag1;		/* flag for invelopt found			*/
   int pflag;		/* flag for param changing			*/
   int pflag1;		/* flag for inpar found			*/
   int tflag;		/* flag for type changing			*/
   int tflag1;		/* flag for intype found			*/
   int vflag;		/* verbose flag					*/

   /* Check for verbosity desired */

   strcpy(verbose,"no");
   vflag = 0;
   fetch("verbose","s",verbose);
   if( (verbose[0] == 'y') || (verbose[0] == 'Y') )
   {
      vflag = 1;
      putch("verbose","s",verbose);
   }

   /* Determine description of input data axes */

   if(!hetch("n1","d",pn1in))
   {
      seperr("Can't find n1 in history \n");
   } 
   if(!hetch("d1","f",pd1in))
   {
      seperr("Can't find d1 in history \n");
   } 
   if(!hetch("o1","f",po1in))
   {
      seperr("Can't find o1 in history \n");
   } 

   *pn2 = 1;
   hetch("n2","d",pn2);

   *pn3 = 1;
   hetch("n3","d",pn3);

   if( (*pn1in) <= 0 )
   {
      seperr("n1 > 0 required on history. \n");
   }
   if( (*pn2) <= 0 )
   {
      seperr("n2 > 0 required on input. \n");
   }
   if( (*pn3) <= 0 )
   {
      seperr("n3 > 0 required on input. \n");
   }
   if( (*pd1in) <= 0.0 )
   {
      seperr("d1 > 0.0 required on history. \n");
   }

   if( vflag )
   {
      puthead("\t\tFound on input: \n");
      putch("n1","d",pn1in);
      putch("n2","d",pn2);
      putch("n3","d",pn3);
      putch("d1","f",pd1in);
      putch("o1","f",po1in);
   }

   /* determine input and output choice of velocity, slowness, or slowness
    * squared.
    */

   *pvopt1 = 2;
   oflag1 = 0;
   if(fetch("invelopt","d",pvopt1))
   {
      oflag1 = 1;
   } 
   else if(hetch("velopt","d",pvopt1))
   {
      oflag1 = 1;
   } 

   *pvopt2 = 2;
   if(fetch("outvelopt","d",pvopt2))
   {
      if( !oflag1 )
      {
         seperr("Can't find invelopt on history or command line\n");
      }
   }
   else
   {
      *pvopt2 = *pvopt1;
      puthead("\t\tOutvelopt not specified; assumed equal to invelopt.\n");
   }

   if( ((*pvopt1) <1) || ((*pvopt1) >4 ))
   {
      seperr("invelopt must be 1,2,3, or 4 \n");
   }
   if( ((*pvopt2) <1) || ((*pvopt2) >4 ))
   {
      seperr("outvelopt must be 1,2,3, or 4 \n");
   }

   if( vflag )
   {
      putch("invelopt","d",pvopt1);
      putch("outvelopt","d",pvopt2);
   }

   /* determine input and output choice of interval or rms */

   *pvtype1 = 1;
   tflag1 = 0;
   if(fetch("intype","d",pvtype1))
   {
      tflag1 = 1;
   } 

   *pvtype2 = 1;
   if(fetch("outtype","d",pvtype2))
   {
      if( !tflag1 )
      {
         seperr("Can't find intype on history or command line\n");
      }
   }
   else
   {
      *pvtype2 = *pvtype1;
      puthead("\t\tOutvtype not specified; assumed equal to intype.\n");
   }

   if( ((*pvtype1) <1) || ((*pvtype1) > 2 ))
   {
      seperr("intype must be 1 or 2 \n");
   }
   if( ((*pvtype2) <1) || ((*pvtype2) > 2 ))
   {
      seperr("outtype must be 1 or 2 \n");
   }

   if( (*pvtype2) != (*pvtype1) )
   {
      if( !(oflag1) )
      {
         seperr("Must specify invelopt if intype != outtype.\n ");
      }
   }

   tflag = 0;
   if( (*pvtype2) != (*pvtype1) )
   {
      tflag = 1; 
   }

   if( vflag )
   {
      putch("intype","d",pvtype1);
      putch("outtype","d",pvtype2);
   }

   /* determine input and output choice of v(z) or v(t) */

   *pvpar1 = 1;
   pflag1 = 0;
   if(fetch("inpar","d",pvpar1))
   {
      pflag1 = 1;
   } 

   *pvpar2 = 1;
   if(fetch("outpar","d",pvpar2))
   {
      if( !pflag1 )
      {
         seperr("Can't find inpar on history or command line\n");
      }
   }
   else
   {
      *pvpar2 = *pvpar1;
      puthead("\t\tOutvparam not specified; assumed equal to inpar.\n");
   }

   if( ((*pvpar1) <1) || ((*pvpar1) > 2 ))
   {
      seperr("inpar must be 1 or 2 \n");
   }
   if( ((*pvpar2) <1) || ((*pvpar2) > 2 ))
   {
      seperr("outpar must be 1 or 2 \n");
   }

   if( (*pvpar2) != (*pvpar1) )
   {
      if( !(oflag1) )
      {
         seperr("Must specify invelopt if inpar != outpar.\n ");
      }
   }

   pflag = 0;
   if( (*pvpar2) != (*pvpar1) )
   {
      pflag = 1; 
   }

   if( vflag )
   {
      putch("inpar","d",pvpar1);
      putch("outpar","d",pvpar2);
   }

   if( (tflag) && (!(pflag1)) )
   {
      seperr("Must specify inpar if intype != outtype.\n");
   }
   if( (pflag) && (!(tflag1)) )
   {
      seperr("Must specify intype if inpar != outpar.\n");
   }

   if( (*pvtype1 == 2) && (*pvpar1 == 2) )
   {
      seperr("intype=2 and inpar=2 (Vrms(z)) is not understood by Vconvert. \n");
   }
   if( (*pvtype2 == 2) && (*pvpar2 == 2) )
   {
      seperr("outtype=2 and outpar=2 (Vrms(z)) is not understood by Vconvert. \n");
   }

   /* Determine output data description */

   *pn1out = (*pn1in);
   *pd1out = (*pd1in);
   *po1out = (*po1in);
   *plsinc = 12;
   *povrsmp = 1;
   *pn1pad = (*pn1in);
   *palpha = 0.01;

   if(pflag)
   {
      if(!getch("n1out","d",pn1out))
      {
         seperr("n1out not found in par file or command line"); 
      }
      if(!getch("d1out","f",pd1out)) 
      {
         seperr("d1out not found in par file or command line"); 
      }
      if(!getch("o1out","f",po1out)) 
      {
         seperr("o1out not found in par file or command line"); 
      }
      if( (*pn1out) <= 0 )
      {
         seperr("n1out > 0 required in parfile or command line. \n");
      }
      if( (*pd1out) <= 0.0 )
      {
         seperr("d1out > 0.0 required in command line or par file. \n");
      }
      putch("n1","d",pn1out); 
      putch("d1","f",pd1out); 
      putch("o1","f",po1out); 

      getch("lsinc","d",plsinc);
      putch("lsinc","d",plsinc);
      if( (*plsinc) < 2)
      {
         seperr("lsinc >= 2 required\n");
      }
      else if ( (*plsinc) %2 != 0)
      {
         (*plsinc) +=1;
         puthead("\t\tlsinc must be even; larger value assumed.\n");
         putch("lsinc","d",plsinc);
      }

      getch("oversamp","d",povrsmp);
      putch("oversamp","d",povrsmp);
      if( (*povrsmp) < 1 )
      {
         seperr("Oversamp >=1 required.\n");
      }

      getch("alpha","f",palpha);
      putch("alpha","f",palpha);
      if( *palpha <= 0.0 )
      {
         seperr("alpha > 0.0 required.\n");
      }
      if( *palpha >= 1.0 )
      {
         seperr("alpha < 1.0 required.\n");
      }

      *ppoints = (*povrsmp) - 1;
      *pn1pad = ( ( (*pn1in)-1 ) * (*povrsmp) ) + 1 ;

   }
   
}































